required_staging_fields = {
    "enable": bool,
    "endpoint": str,
    "secrets_dir": str,
    "input_path": str,
    "output_path": str,
}

required_staging_values = {
    "enable": True,
    "endpoint": True,
    "secrets_dir": False,
    "input_path": False,
    "output_path": True,
}
